function [fn,fmedian,qn,qmedian,gam] = time_warping_median(f,t)

lambda=0;
if isempty(lambda)
    lambda = 0;
end

[M, N] = size(f);
f0 =f;
figs=1;
if figs==1
    figure(100); clf;
    plot(t, f0, 'linewidth', 2);
    xlim([t(1) t(end)]);
    ylim auto;
    set(gca,'FontSize',19)
    ti = get(gca,'TightInset');
    set(gca,'Position',[ti(1) ti(2) 1-ti(3)-ti(1) 1-ti(4)-ti(2)]);
    pause(0.1);
end

% compute q-functions
binsize = mean(diff(t));
for i = 1:N
    q(:,i) = sign(gradient(f(:,i), binsize)).*sqrt(abs(gradient(f(:,i), binsize)));
    fn(:,i) = f(1,i)+cumtrapz(t,q(:,i).*abs(q(:,i)));
end

%%% set initial 
disp(sprintf('\n Initializing...\n'));
mnq = mean(q,2);    % mean of rows
dqq = sqrt(sum((q - mnq*ones(1,N)).^2,1));
[~, min_ind] = min(dqq);
mq = q(:,min_ind); 

for k = 1:N
    q_c = q(:,k,1)'; mq_c = mq';
    gam0 = DynamicProgrammingQ(q_c,mq_c,lambda,0);
    gam(k,:) = (gam0-gam0(1))/(gam0(end)-gam0(1));  % slight change on scale
end
gamI = SqrtMeanInverse(gam);
gamI_dev = gradient(gamI, 1/(M-1));
mq = interp1(t, mq, (t(end)-t(1)).*gamI + t(1))'.*sqrt(gamI_dev');

%%% compute median
disp(sprintf(' Computing Karcher median of %d functions in SRVF space...\n',N));
ds = inf;
stp=.3;
MaxItr = 20;
for r = 1:MaxItr
    disp(sprintf('updating step: r=%d', r)); 
    
    %%%% Matching Step %%%%
    clear gam gam_dev;
    % use DP to find the optimal warping for each function w.r.t. median
    for k = 1:N
        q_c = q(:,k,1)'; mq_c = mq(:,r)';
        gam0 = DynamicProgrammingQ(q_c/inner_product(q_c, q_c),mq_c/inner_product(mq_c, mq_c),lambda,0);
        gam(k,:) = (gam0-gam0(1))/(gam0(end)-gam0(1));  % slight change on scale
        gam_dev(k,:) = gradient(gam(k,:), 1/(M-1));
        q(:,k,r+1) = interp1(t, q(:,k,1), (t(end)-t(1)).*gam(k,:) + t(1))'.*sqrt(gam_dev(k,:)');
        f(:,k,r+1) = interp1(t, f(:,k,1), (t(end)-t(1)).*gam(k,:) + t(1))';
        v=q(:,k,r+1)-mq(:,r);
        d(k)=sqrt(trapz(t,v.^2));
        vtil(:,k)=v/d(k);
        dtil(:,k)=1/d(k);    
    end
    
    ds(r+1) = sum(trapz(t, (mq(:,r)*ones(1,N)-q(:,:,r+1)).^2)) + ...
                lambda*sum(trapz(t, (1-sqrt(gam_dev')).^2));
    
    %%%% Minimization Step %%%%
    % compute the mean of the matched function
    vbar=sum(vtil,2)*sum(dtil)^(-1);
    mq(:,r+1) = mq(:,r)+stp*vbar;
    
    qun(r) = norm(mq(:,r+1)-mq(:,r))/norm(mq(:,r));
    mf = median(f0(1,:))+cumtrapz(t,mq(:,r+1).*abs(mq(:,r+1)));
    if (qun(r) < 1e-10)
        break;
    end
end

if lambda == 0
    disp(sprintf('additional run when lambda = 0')); 
    r = r+1;
    for k = 1:N
        q_c = q(:,k,1)'; mq_c = mq(:,r)';
        gam0 = DynamicProgrammingQ(q_c,mq_c,lambda,0);
        gam(k,:) = (gam0-gam0(1))/(gam0(end)-gam0(1));  % slight change on scale
        gam_dev(k,:) = gradient(gam(k,:), 1/(M-1));
        q(:,k,r+1) = interp1(t, q(:,k,1), (t(end)-t(1)).*gam(k,:) + t(1))'.*sqrt(gam_dev(k,:)');
        f(:,k,r+1) = interp1(t, f(:,k,1), (t(end)-t(1)).*gam(k,:) + t(1))';  
    end
    ds(r+1) = sum(trapz(t, (mq(:,r)*ones(1,N)-q(:,:,r+1)).^2)) + ...
                lambda*sum(trapz(t, (1-sqrt(gam_dev')).^2));
            
      gamI = SqrtMeanInverse(gam);
      gamI_dev = gradient(gamI, 1/(M-1));
      mq(:,r+1) = interp1(t, mq(:,r), (t(end)-t(1)).*gamI + t(1))'.*sqrt(gamI_dev');
      for k = 1:N
        q_c = q(:,k,1)'; mq_c = mq(:,r+1)';
        gam0 = DynamicProgrammingQ(q_c,mq_c,lambda,0);
        gam(k,:) = (gam0-gam0(1))/(gam0(end)-gam0(1));  % slight change on scale
        gam_dev(k,:) = gradient(gam(k,:), 1/(M-1));
        q(:,k,r+1) = interp1(t, q(:,k,1), (t(end)-t(1)).*gam(k,:) + t(1))'.*sqrt(gam_dev(k,:)');
        f(:,k,r+1) = interp1(t, f(:,k,1), (t(end)-t(1)).*gam(k,:) + t(1))';  
    end
end

% aligned data
qn = q(:,:,r+1);
fn = f(:,:,r+1);
fn=fn-repmat(fn(1,:),size(fn,1),1);
qmedian = mq(:,r+1);
fmedian = cumtrapz(t,qmedian.*abs(qmedian));

if figs==1
    figure(300); clf;
    plot(t, fn, 'LineWidth',2);
    xlim([t(1) t(end)]);
    ylim auto;
    set(gca,'FontSize',19);
    figax=axis;
    ti = get(gca,'TightInset');
    set(gca,'Position',[ti(1) ti(2) 1-ti(3)-ti(1) 1-ti(4)-ti(2)]);
    
    figure(400); clf;
    plot(linspace(0,1,M), gam', 'linewidth', 2);
    axis square;
    set(gca,'FontSize',19);
    ti = get(gca,'TightInset');
    set(gca,'Position',[ti(1) ti(2) 1-ti(3)-ti(1) 1-ti(4)-ti(2)]);
end